<?php

declare(strict_types=1);

namespace Drupal\Tests\views_better_rest\Kernel\Views;

use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\rest\Plugin\views\display\RestExport;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\Tests\views\Kernel\ViewsKernelTestBase;
use Drupal\views\Entity\View;

/**
 * Tests the Better REST export view plugins.
 *
 * @group views_better_rest
 */
class BetterRestExportTest extends ViewsKernelTestBase {
  use UserCreationTrait;

  /**
    * The variable.
    *
    * @var mixed
  */
  protected $strictConfigSchema = FALSE;

  /**
   * A regular user with 'access content' permission.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $user;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'views_better_rest',
    'node',
    'rest',
    'serialization',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp($import_test_views = TRUE): void {
    parent::setUp($import_test_views);
    $this->installEntitySchema('node');
    $this->installEntitySchema('user');
    $this->installSchema('node', ['node_access']);
    $this->installConfig(['views_better_rest', 'node', 'rest', 'user']);

    // Create testing content types.
    $node_type_example = NodeType::create([
      'type' => 'example',
      'name' => 'Example',
    ]);
    $node_type_example->save();

    // Create testing node.
    $node = Node::create([
      'type' => 'example',
      'title' => 'Foo',
    ]);
    $node->save();

    // Create and log in testing user with necessary permission.
    $this->setCurrentUser($this->createUser(['access content']));
  }

  /**
   * Tests better_rest_resources_serializer response with exposed filters, sorts and pager.
   */
  public function testBetterRESTSerializerResponse(): void {
    $response = RestExport::buildResponse('content_rest', 'better_rest_export_1', []);
    $response_content = json_decode($response->getContent(), TRUE);

    // Tests "endpoint" output.
    $this->assertEquals(
      [
        'path' => 'api/node',
        'args' => [],
        'requested' => '/api/node',
      ],
      $response_content['endpoint'],
      'Expected "endpoint" output'
    );

    // Tests "exposed_filters" output available.
    $this->assertEquals('Content type', $response_content['exposed_filters'][0]['label'], 'Expected "exposed_filters" output');

    // Tests "exposed_sorts" output available.
    $this->assertEquals('Authored on', $response_content['exposed_sorts'][0]['label'], 'Expected "exposed_sorts" output');

    // Tests "pager" output when pager enabled.
    $this->assertEquals(TRUE, $response_content['pager']['active'], 'Expected pager output');

    // Tests "rows" output not available.
    $this->assertTrue(is_array($response_content['rows']) && count($response_content['rows']) === 1, 'Expected rows key is available');
  }
  
}
